package com.qmth.distributed.print.api;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.dto.CourseWeightDto;
import com.qmth.distributed.print.business.bean.dto.obe.CourseRequirementMatrixDto;
import com.qmth.distributed.print.business.bean.dto.obe.MatrixRequirementDto;
import com.qmth.distributed.print.business.bean.dto.obe.MatrixSubRequirementDto;
import com.qmth.distributed.print.business.bean.dto.report.*;
import com.qmth.distributed.print.business.bean.result.ObeCourseWeightResult;
import com.qmth.distributed.print.business.bean.result.ScoreResult;
import com.qmth.distributed.print.business.entity.TRBasicInfo;
import com.qmth.distributed.print.business.service.TRBasicInfoService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * obe专业达成度报表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "obe专业达成度报表Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_OBE)
public class ObeReportController {
    private final static Logger log = LoggerFactory.getLogger(ObeReportController.class);

    @Resource
    TRBasicInfoService trBasicInfoService;

    @ApiOperation(value = "毕业要求达成度雷达报表")
    @RequestMapping(value = "/requirements/radar/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度雷达报表", response = ScoreResult.class)})
    public Result requirementsRadarReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId,
                                          @ApiParam(value = "专业id", required = true) @RequestParam Long profession) {
//        String s = "[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"cultureProgramId\":null,\"obeCourseId\":null,\"obeRequirementId\":null,\"obeSubRequirementId\":null,\"scale\":null,\"intensity\":null,\"courseCode\":\"1400\",\"courseName\":\"微积分\",\"courseSortNum\":0,\"requirementSortNum\":0,\"subRequirementSortNum\":0,\"requirementContent\":null,\"canEdit\":true,\"requirements\":[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程知识\",\"orgId\":null,\"content\":\"能够将数学、自然科学、工程基础和专业知识用于解决复杂工程问题\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077696\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"1-1\",\"orgId\":null,\"scale\":0.1,\"content\":\"指标点1-1-content\"}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"问题分析\",\"orgId\":null,\"content\":\"能够应用数学、自然科学和工程科学的基本原理，识别、表达、并通过文献研究分析复杂工程问题，以获得有效结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077697\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"2-1\",\"orgId\":null,\"scale\":0.2,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"设计/开发解决方案\",\"orgId\":null,\"content\":\"能够设计针对复杂工程问题的解决方案，设计满足特定需求的系统、单元（部件）或工艺流程，并能够在设计环节中体现创新意识，考虑社会、健康、安全、法律、文化以及环境等因素\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077698\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"3-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"研究\",\"orgId\":null,\"content\":\"能够基于科学原理并采用科学方法对复杂工程问题进行研究，包括设计实验、分析与解释数据、并通过信息综合得到合理有效的结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077699\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"4-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"使用工具\",\"orgId\":null,\"content\":\"能够针对复杂工程问题，开发、选择与使用恰当的技术、资源、现代工程工具和信息技术工具，包括对复杂工程问题的预测与模拟，并能够理解其局限性\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077700\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"5-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程与社会\",\"orgId\":null,\"content\":\"能够基于工程相关背景知识进行合理分析，评价专业工程实践和复杂工程问题解决方案对社会、健康、安全、法律以及文化的影响，并理解应承担的责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077701\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"6-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"环境和可持续发展\",\"orgId\":null,\"content\":\"能够理解和评价针对复杂工程问题的工程实践对环境、社会可持续发展的影响\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077702\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"7-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"职业规范\",\"orgId\":null,\"content\":\"具有人文社会科学素养、社会责任感，能够在工程实践中理解并遵守工程职业道德和规范，履行责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077703\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"8-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"个人与团队\",\"orgId\":null,\"content\":\"能够在多学科背景下的团队中承担个体、团队成员以及负责人的角色\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077704\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"9-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"沟通\",\"orgId\":null,\"content\":\"能够就复杂工程问题与业界同行及社会公众进行有效沟通和交流，包括撰写报告和设计文稿、陈述发言、清晰表达或回应指令。并具备一定的国际视野，能够在跨文化背景下进行沟通和交流\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077705\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"10-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"项目管理\",\"orgId\":null,\"content\":\"理解并掌握工程管理原理与经济决策方法，并能在多学科环境中应用\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077706\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"11-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"终身学习\",\"orgId\":null,\"content\":\"具有自主学习和终身学习的意识，有不断学习和适应发展的能力\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077707\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"12-1\",\"orgId\":null,\"scale\":null,\"content\":null}]}]}]";
        String s = "[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"cultureProgramId\":null,\"obeCourseId\":null,\"obeRequirementId\":null,\"obeSubRequirementId\":null,\"scale\":null,\"intensity\":null,\"courseCode\":\"1400\",\"courseName\":\"微积分\",\"courseSortNum\":0,\"requirementSortNum\":0,\"subRequirementSortNum\":0,\"requirementContent\":null,\"canEdit\":true,\"requirements\":[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程知识\",\"orgId\":null,\"content\":\"能够将数学、自然科学、工程基础和专业知识用于解决复杂工程问题\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077696\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"1-1\",\"orgId\":null,\"scale\":0.1,\"content\":\"指标点1-1-content\"}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"问题分析\",\"orgId\":null,\"content\":\"能够应用数学、自然科学和工程科学的基本原理，识别、表达、并通过文献研究分析复杂工程问题，以获得有效结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077697\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"2-1\",\"orgId\":null,\"scale\":0.2,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"设计/开发解决方案\",\"orgId\":null,\"content\":\"能够设计针对复杂工程问题的解决方案，设计满足特定需求的系统、单元（部件）或工艺流程，并能够在设计环节中体现创新意识，考虑社会、健康、安全、法律、文化以及环境等因素\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077698\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"3-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"研究\",\"orgId\":null,\"content\":\"能够基于科学原理并采用科学方法对复杂工程问题进行研究，包括设计实验、分析与解释数据、并通过信息综合得到合理有效的结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077699\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"4-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"使用工具\",\"orgId\":null,\"content\":\"能够针对复杂工程问题，开发、选择与使用恰当的技术、资源、现代工程工具和信息技术工具，包括对复杂工程问题的预测与模拟，并能够理解其局限性\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077700\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"5-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程与社会\",\"orgId\":null,\"content\":\"能够基于工程相关背景知识进行合理分析，评价专业工程实践和复杂工程问题解决方案对社会、健康、安全、法律以及文化的影响，并理解应承担的责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077701\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"6-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"环境和可持续发展\",\"orgId\":null,\"content\":\"能够理解和评价针对复杂工程问题的工程实践对环境、社会可持续发展的影响\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077702\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"7-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"职业规范\",\"orgId\":null,\"content\":\"具有人文社会科学素养、社会责任感，能够在工程实践中理解并遵守工程职业道德和规范，履行责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077703\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"8-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"个人与团队\",\"orgId\":null,\"content\":\"能够在多学科背景下的团队中承担个体、团队成员以及负责人的角色\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077704\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"9-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"沟通\",\"orgId\":null,\"content\":\"能够就复杂工程问题与业界同行及社会公众进行有效沟通和交流，包括撰写报告和设计文稿、陈述发言、清晰表达或回应指令。并具备一定的国际视野，能够在跨文化背景下进行沟通和交流\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077705\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"10-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"项目管理\",\"orgId\":null,\"content\":\"理解并掌握工程管理原理与经济决策方法，并能在多学科环境中应用\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077706\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"11-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"终身学习\",\"orgId\":null,\"content\":\"具有自主学习和终身学习的意识，有不断学习和适应发展的能力\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077707\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"12-1\",\"orgId\":null,\"scale\":null,\"content\":null}]}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"cultureProgramId\":null,\"obeCourseId\":null,\"obeRequirementId\":null,\"obeSubRequirementId\":null,\"scale\":null,\"intensity\":null,\"courseCode\":\"1400\",\"courseName\":\"微积分\",\"courseSortNum\":0,\"requirementSortNum\":0,\"subRequirementSortNum\":0,\"requirementContent\":null,\"canEdit\":true,\"requirements\":[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程知识\",\"orgId\":null,\"content\":\"能够将数学、自然科学、工程基础和专业知识用于解决复杂工程问题\",\"expectValue\":0.2,\"subRequirements\":[{\"id\":\"533285303218077696\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"1-1\",\"orgId\":null,\"scale\":0.3,\"content\":\"指标点1-1-content\"}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"问题分析\",\"orgId\":null,\"content\":\"能够应用数学、自然科学和工程科学的基本原理，识别、表达、并通过文献研究分析复杂工程问题，以获得有效结论\",\"expectValue\":0.4,\"subRequirements\":[{\"id\":\"533285303218077697\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"2-1\",\"orgId\":null,\"scale\":0.4,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"设计/开发解决方案\",\"orgId\":null,\"content\":\"能够设计针对复杂工程问题的解决方案，设计满足特定需求的系统、单元（部件）或工艺流程，并能够在设计环节中体现创新意识，考虑社会、健康、安全、法律、文化以及环境等因素\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077698\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"3-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"研究\",\"orgId\":null,\"content\":\"能够基于科学原理并采用科学方法对复杂工程问题进行研究，包括设计实验、分析与解释数据、并通过信息综合得到合理有效的结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077699\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"4-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"使用工具\",\"orgId\":null,\"content\":\"能够针对复杂工程问题，开发、选择与使用恰当的技术、资源、现代工程工具和信息技术工具，包括对复杂工程问题的预测与模拟，并能够理解其局限性\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077700\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"5-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程与社会\",\"orgId\":null,\"content\":\"能够基于工程相关背景知识进行合理分析，评价专业工程实践和复杂工程问题解决方案对社会、健康、安全、法律以及文化的影响，并理解应承担的责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077701\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"6-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"环境和可持续发展\",\"orgId\":null,\"content\":\"能够理解和评价针对复杂工程问题的工程实践对环境、社会可持续发展的影响\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077702\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"7-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"职业规范\",\"orgId\":null,\"content\":\"具有人文社会科学素养、社会责任感，能够在工程实践中理解并遵守工程职业道德和规范，履行责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077703\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"8-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"个人与团队\",\"orgId\":null,\"content\":\"能够在多学科背景下的团队中承担个体、团队成员以及负责人的角色\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077704\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"9-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"沟通\",\"orgId\":null,\"content\":\"能够就复杂工程问题与业界同行及社会公众进行有效沟通和交流，包括撰写报告和设计文稿、陈述发言、清晰表达或回应指令。并具备一定的国际视野，能够在跨文化背景下进行沟通和交流\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077705\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"10-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"项目管理\",\"orgId\":null,\"content\":\"理解并掌握工程管理原理与经济决策方法，并能在多学科环境中应用\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077706\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"11-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"终身学习\",\"orgId\":null,\"content\":\"具有自主学习和终身学习的意识，有不断学习和适应发展的能力\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077707\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"12-1\",\"orgId\":null,\"scale\":null,\"content\":null}]}]}]";
        List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList = JSONArray.parseArray(s, CourseRequirementMatrixDto.class);
        log.info("courseRequirementMatrixDto:{}", JacksonUtil.parseJson(courseRequirementMatrixDtoList));
        Long examId = 510450135071723520L;
        Long obeCourseOutlineId = 533296474771947520L;
        Map<String, Object> map = this.matrixCourseReportInfo(courseRequirementMatrixDtoList, obeCourseOutlineId, examId);
        ObeCourseRequirementDto obeCourseRequirementResultDto = null;
        if (!org.springframework.util.CollectionUtils.isEmpty(map)) {
            List<ObeCourseRequirementDto> obeCourseRequirementDtoList = (List<ObeCourseRequirementDto>) map.get(SystemConstant.OBE_COURSE_REQUIREMENT_DTO_LIST);
            Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap = (Map<Long, List<ObeSubRequirementsDto>>) map.get(SystemConstant.OBE_SUB_REQUIREMENT_DTO_MAP);
            obeCourseRequirementResultDto = this.matrixCourseReport(obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
        }
        return ResultUtil.ok(obeCourseRequirementResultDto);
    }

    @ApiOperation(value = "毕业要求达成度详情报表")
    @RequestMapping(value = "/requirements/detail/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度详情报表", response = ScoreResult.class)})
    public Result requirementsDetailReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId,
                                           @ApiParam(value = "专业id", required = true) @RequestParam Long profession) {
        String s = "[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"cultureProgramId\":null,\"obeCourseId\":null,\"obeRequirementId\":null,\"obeSubRequirementId\":null,\"scale\":null,\"intensity\":null,\"courseCode\":\"1400\",\"courseName\":\"微积分\",\"courseSortNum\":0,\"requirementSortNum\":0,\"subRequirementSortNum\":0,\"requirementContent\":null,\"canEdit\":true,\"requirements\":[{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程知识\",\"orgId\":null,\"content\":\"能够将数学、自然科学、工程基础和专业知识用于解决复杂工程问题\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077696\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"1-1\",\"orgId\":null,\"scale\":0.1,\"content\":\"指标点1-1-content\"}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"问题分析\",\"orgId\":null,\"content\":\"能够应用数学、自然科学和工程科学的基本原理，识别、表达、并通过文献研究分析复杂工程问题，以获得有效结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077697\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"2-1\",\"orgId\":null,\"scale\":0.2,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"设计/开发解决方案\",\"orgId\":null,\"content\":\"能够设计针对复杂工程问题的解决方案，设计满足特定需求的系统、单元（部件）或工艺流程，并能够在设计环节中体现创新意识，考虑社会、健康、安全、法律、文化以及环境等因素\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077698\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"3-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"研究\",\"orgId\":null,\"content\":\"能够基于科学原理并采用科学方法对复杂工程问题进行研究，包括设计实验、分析与解释数据、并通过信息综合得到合理有效的结论\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077699\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"4-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"使用工具\",\"orgId\":null,\"content\":\"能够针对复杂工程问题，开发、选择与使用恰当的技术、资源、现代工程工具和信息技术工具，包括对复杂工程问题的预测与模拟，并能够理解其局限性\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077700\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"5-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"工程与社会\",\"orgId\":null,\"content\":\"能够基于工程相关背景知识进行合理分析，评价专业工程实践和复杂工程问题解决方案对社会、健康、安全、法律以及文化的影响，并理解应承担的责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077701\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"6-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"环境和可持续发展\",\"orgId\":null,\"content\":\"能够理解和评价针对复杂工程问题的工程实践对环境、社会可持续发展的影响\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077702\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"7-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"职业规范\",\"orgId\":null,\"content\":\"具有人文社会科学素养、社会责任感，能够在工程实践中理解并遵守工程职业道德和规范，履行责任\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077703\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"8-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"个人与团队\",\"orgId\":null,\"content\":\"能够在多学科背景下的团队中承担个体、团队成员以及负责人的角色\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077704\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"9-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"沟通\",\"orgId\":null,\"content\":\"能够就复杂工程问题与业界同行及社会公众进行有效沟通和交流，包括撰写报告和设计文稿、陈述发言、清晰表达或回应指令。并具备一定的国际视野，能够在跨文化背景下进行沟通和交流\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077705\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"10-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"项目管理\",\"orgId\":null,\"content\":\"理解并掌握工程管理原理与经济决策方法，并能在多学科环境中应用\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077706\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"11-1\",\"orgId\":null,\"scale\":null,\"content\":null}]},{\"id\":null,\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"终身学习\",\"orgId\":null,\"content\":\"具有自主学习和终身学习的意识，有不断学习和适应发展的能力\",\"expectValue\":0.1,\"subRequirements\":[{\"id\":\"533285303218077707\",\"createId\":null,\"createTime\":null,\"updateId\":null,\"updateTime\":null,\"schoolId\":null,\"name\":\"12-1\",\"orgId\":null,\"scale\":null,\"content\":null}]}]}]";
        List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList = JSONArray.parseArray(s, CourseRequirementMatrixDto.class);
        log.info("courseRequirementMatrixDto:{}", JacksonUtil.parseJson(courseRequirementMatrixDtoList));
        Long examId = 510450135071723520L;
        Long obeCourseOutlineId = 533296474771947520L;
        Map<String, Object> map = this.matrixCourseReportInfo(courseRequirementMatrixDtoList, obeCourseOutlineId, examId);
        List<ObeCourseRequirementDto> obeCourseRequirementDtoList = null;
        if (!org.springframework.util.CollectionUtils.isEmpty(map)) {
            obeCourseRequirementDtoList = (List<ObeCourseRequirementDto>) map.get(SystemConstant.OBE_COURSE_REQUIREMENT_DTO_LIST);
            Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap = (Map<Long, List<ObeSubRequirementsDto>>) map.get(SystemConstant.OBE_SUB_REQUIREMENT_DTO_MAP);
            obeCourseRequirementDtoList = this.matrixCourseReportCommon(obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
        }
        return ResultUtil.ok(obeCourseRequirementDtoList);
    }

    @ApiOperation(value = "毕业要求达成度学生报表")
    @RequestMapping(value = "/requirements/student/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度学生报表", response = ScoreResult.class)})
    public Result requirementsStudentReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId) {
        return ResultUtil.ok(true);
    }

    /**
     * 换算课程毕业达成度报表详情
     *
     * @param courseRequirementMatrixDtoList
     * @param obeCourseOutlineId
     * @param examId
     * @return
     */
    protected Map<String, Object> matrixCourseReportInfo(List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList,
                                                         Long obeCourseOutlineId, Long examId) {
        Map<String, Object> resultMap = new HashMap<>();
        List<ObeCourseRequirementDto> obeCourseRequirementDtoList = new ArrayList<>();
        Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap = new LinkedHashMap<>();
        if (!CollectionUtils.isEmpty(courseRequirementMatrixDtoList)) {
            ObeCourseWeightResult obeCourseWeightResult = trBasicInfoService.findCourseWeightResultRmi(obeCourseOutlineId);
            String s1 = "{\"weightSettingSign\":\"531143129521197056\",\"dimensionSign\":\"511110362758647808\",\"submitForm\":[{\"courseTargetId\":\"511110308371107840\",\"courseTargetName\":\"课程目标1\",\"obeCultureProgramRequirementId\":533285303218077696,\"obeCultureProgramRequirementName\":\"1-1\",\"dimensionList\":[{\"id\":\"511110237860663297\",\"code\":\"A1\",\"name\":\"第一节 映射与函数\"}],\"dimensionStr\":\"第一节 映射与函数(A1)\",\"degreeRequirement\":\"课程目标1\",\"totalWeight\":40,\"evaluationList\":[{\"evaluationId\":\"511110589855043584\",\"evaluationName\":\"评价方式2\",\"evaluationDesc\":\"评价方式2\",\"enable\":true,\"weight\":10,\"targetScore\":100,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"511110641155575808\",\"evaluationName\":\"评价方式1\",\"evaluationDesc\":\"评价方式1\",\"enable\":true,\"weight\":10,\"targetScore\":100,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"511110668787650560\",\"evaluationName\":\"评价方式3\",\"evaluationDesc\":\"评价方式3\",\"enable\":false,\"weight\":null,\"targetScore\":0,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"511110692196061184\",\"evaluationName\":\"评价方式4\",\"evaluationDesc\":\"评价方式14\",\"enable\":false,\"weight\":null,\"targetScore\":0,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"510826411045425152\",\"evaluationName\":\"期末考试\",\"evaluationDesc\":\"按阅卷功能模块里的卷面成绩计算\",\"enable\":true,\"weight\":20,\"targetScore\":50,\"evaluationType\":\"DEFAULT\"}]},{\"courseTargetId\":\"511110362817368064\",\"courseTargetName\":\"课程目标2\",\"obeCultureProgramRequirementId\":533285303218077697,\"obeCultureProgramRequirementName\":\"2-1\",\"dimensionList\":[{\"id\":\"511110237860663299\",\"code\":\"B1\",\"name\":\"第二节 数列的极限\"}],\"dimensionStr\":\"第二节 数列的极限(B1)\",\"degreeRequirement\":\"课程目标2\",\"totalWeight\":60,\"evaluationList\":[{\"evaluationId\":\"511110589855043584\",\"evaluationName\":\"评价方式2\",\"evaluationDesc\":\"评价方式2\",\"enable\":false,\"weight\":null,\"targetScore\":0,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"511110641155575808\",\"evaluationName\":\"评价方式1\",\"evaluationDesc\":\"评价方式1\",\"enable\":false,\"weight\":null,\"targetScore\":0,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"511110668787650560\",\"evaluationName\":\"评价方式3\",\"evaluationDesc\":\"评价方式3\",\"enable\":true,\"weight\":40,\"targetScore\":100,\"evaluationType\":\"CUSTOM\"},{\"evaluationId\":\"510826411045425152\",\"evaluationName\":\"期末考试\",\"evaluationDesc\":\"按阅卷功能模块里的卷面成绩计算\",\"enable\":true,\"weight\":20,\"targetScore\":50,\"evaluationType\":\"DEFAULT\"}]}]}";
            obeCourseWeightResult = JSONObject.parseObject(s1, ObeCourseWeightResult.class);
            log.info("obeCourseWeightResult:{}", JacksonUtil.parseJson(obeCourseWeightResult));

            List<CourseWeightDto> courseWeightDtoList = obeCourseWeightResult.getSubmitForm();
            //课程目标指标点集合map
            Map<Long, CourseWeightDto> cultureProgramRequirementIdMap = courseWeightDtoList.stream().collect(Collectors.toMap(CourseWeightDto::getObeCultureProgramRequirementId, Function.identity(), (dto1, dto2) -> dto1));
            Map<Long, List<ObeSubRequirementsDto>> matrixSubRequirementIdMap = new HashMap<>();//课程目标指标点集合-多个目标共用一个指标点
            Map<Long, CourseTargetWebDto> courseTargetWebDtoMap = new HashMap<>();//课程目标集合
            for (CourseRequirementMatrixDto c : courseRequirementMatrixDtoList) {
                ObeCourseRequirementDto obeCourseRequirementDto = new ObeCourseRequirementDto(c);
                obeCourseRequirementDtoList.add(obeCourseRequirementDto);
                TRBasicInfo trBasicInfo = trBasicInfoService.queryBasicInfo(examId, c.getCourseCode(), null, 510826410978316288L);
                if (Objects.nonNull(trBasicInfo)) {
                    ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = Objects.nonNull(trBasicInfo.getCourseEvaluationResult()) ? JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class) : null;
                    if (Objects.nonNull(reportCourseEvaluationResultDto)) {
                        List<CourseTargetWebDto> courseTargetWebDtoList = reportCourseEvaluationResultDto.getTargetList();
                        courseTargetWebDtoMap = courseTargetWebDtoList.stream().collect(Collectors.toMap(CourseTargetWebDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
                    }
                }
                List<MatrixRequirementDto> matrixRequirementDtoList = c.getRequirements();
                if (!CollectionUtils.isEmpty(matrixRequirementDtoList)) {
                    List<ObeRequirementDto> obeRequirementDtoList = new ArrayList<>();
                    obeCourseRequirementDto.setObeRequirements(obeRequirementDtoList);
                    for (MatrixRequirementDto matrixRequirementDto : matrixRequirementDtoList) {
                        ObeRequirementDto obeRequirementDto = new ObeRequirementDto(matrixRequirementDto);
                        obeRequirementDtoList.add(obeRequirementDto);
                        List<MatrixSubRequirementDto> matrixSubRequirementDtoList = matrixRequirementDto.getSubRequirements();
                        if (!CollectionUtils.isEmpty(matrixSubRequirementDtoList)) {
                            List<ObeSubRequirementsDto> obeSubRequirementsDtoList = new ArrayList<>();
                            obeRequirementDto.setObeSubRequirements(obeSubRequirementsDtoList);
                            for (MatrixSubRequirementDto matrixSubRequirementDto : matrixSubRequirementDtoList) {
                                ObeSubRequirementsDto obeSubRequirementsDto = new ObeSubRequirementsDto(matrixSubRequirementDto);
                                obeSubRequirementsDtoList.add(obeSubRequirementsDto);
                                if (Objects.nonNull(matrixSubRequirementDto.getScale())) {
                                    //多个课程目标关联同一个指标点
                                    if (matrixSubRequirementIdMap.containsKey(matrixSubRequirementDto.getId())) {
                                        List<ObeSubRequirementsDto> obeRequirementsDtoList = matrixSubRequirementIdMap.get(matrixSubRequirementDto.getId());
                                        Map<Long, CourseTargetWebDto> map = obeRequirementsDtoList.stream().collect(Collectors.toMap(ObeSubRequirementsDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
                                        CourseWeightDto courseWeightDto = cultureProgramRequirementIdMap.get(matrixSubRequirementDto.getId());
                                        if (Objects.nonNull(courseWeightDto)) {
                                            CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseWeightDto.getCourseTargetId());
                                            if (!map.containsKey(courseTargetWebDto.getTargetId())) {
                                                obeRequirementsDtoList.add(new ObeSubRequirementsDto(courseTargetWebDto, matrixSubRequirementDto));
                                            }
                                        }
                                        matrixSubRequirementIdMap.put(matrixSubRequirementDto.getId(), obeRequirementsDtoList);
                                    } else {
                                        CourseWeightDto courseWeightDto = cultureProgramRequirementIdMap.get(matrixSubRequirementDto.getId());
                                        if (Objects.nonNull(courseWeightDto)) {
                                            CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseWeightDto.getCourseTargetId());
                                            matrixSubRequirementIdMap.put(matrixSubRequirementDto.getId(), new ArrayList<>(Arrays.asList(new ObeSubRequirementsDto(courseTargetWebDto, matrixSubRequirementDto))));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                //计算各个课程的指标点达成值
                if (!org.springframework.util.CollectionUtils.isEmpty(matrixSubRequirementIdMap)) {
                    for (Map.Entry<Long, List<ObeSubRequirementsDto>> entry : matrixSubRequirementIdMap.entrySet()) {
                        Double evaluationValue = entry.getValue().stream().mapToDouble(x -> x.getEvaluationValue().doubleValue()).average().orElse(0.0d);
                        ObeSubRequirementsDto obeSubRequirementsDto = entry.getValue().get(0);
                        BigDecimal bigDecimal = new BigDecimal(evaluationValue).multiply(new BigDecimal(obeSubRequirementsDto.getScale())).setScale(2, BigDecimal.ROUND_HALF_UP);
                        entry.getValue().stream().peek(x -> x.setMatrixDegree(bigDecimal)).collect(Collectors.toList());
                    }
                }
                //计算各个课程的指标点达成值
                List<ObeRequirementDto> obeRequirementDtoList = obeCourseRequirementDto.getObeRequirements();
                if (!CollectionUtils.isEmpty(obeRequirementDtoList)) {
                    for (ObeRequirementDto o : obeRequirementDtoList) {
                        List<ObeSubRequirementsDto> obeSubRequirementsDtoList = o.getObeSubRequirements();
                        for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                            if (matrixSubRequirementIdMap.containsKey(obeSubRequirementsDto.getId())) {
                                obeSubRequirementsDto.setMatrixDegree(matrixSubRequirementIdMap.get(obeSubRequirementsDto.getId()).get(0).getMatrixDegree());
                            }
                            if (obeSubRequirementsDtoMap.containsKey(obeSubRequirementsDto.getId())) {
                                List<ObeSubRequirementsDto> obeSubRequirementsDtoListTemp = obeSubRequirementsDtoMap.get(obeSubRequirementsDto.getId());
                                obeSubRequirementsDtoListTemp.add(obeSubRequirementsDto);
                                obeSubRequirementsDtoMap.put(obeSubRequirementsDto.getId(), obeSubRequirementsDtoListTemp);
                            } else {
                                obeSubRequirementsDtoMap.put(obeSubRequirementsDto.getId(), new ArrayList<>(Arrays.asList(obeSubRequirementsDto)));
                            }
                        }
                    }
                }
            }
        }
        resultMap.put(SystemConstant.OBE_COURSE_REQUIREMENT_DTO_LIST, obeCourseRequirementDtoList);
        resultMap.put(SystemConstant.OBE_SUB_REQUIREMENT_DTO_MAP, obeSubRequirementsDtoMap);
        return resultMap;
    }

    /**
     * 换算课程毕业达成度报表
     *
     * @param obeCourseRequirementDtoList
     * @param obeSubRequirementsDtoMap
     * @return
     */
    protected ObeCourseRequirementDto matrixCourseReport(List<ObeCourseRequirementDto> obeCourseRequirementDtoList,
                                                         Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        //计算指标和毕业达成度
        ObeCourseRequirementDto obeCourseRequirementResultDto = new ObeCourseRequirementDto();
        if (!CollectionUtils.isEmpty(obeCourseRequirementDtoList)) {
            obeCourseRequirementDtoList = this.matrixCourseReportCommon(obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
            obeCourseRequirementResultDto = obeCourseRequirementDtoList.get(obeCourseRequirementDtoList.size() - 1);
            obeCourseRequirementResultDto.updateInfo();
        }
        return obeCourseRequirementResultDto;
    }

    /**
     * 换算课程毕业达成度报表共用
     *
     * @param obeCourseRequirementDtoList
     * @param obeSubRequirementsDtoMap
     * @return
     */
    protected List<ObeCourseRequirementDto> matrixCourseReportCommon(List<ObeCourseRequirementDto> obeCourseRequirementDtoList,
                                                                     Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        for (ObeCourseRequirementDto o : obeCourseRequirementDtoList) {
            List<ObeRequirementDto> obeRequirementDtoList = o.getObeRequirements();
            for (ObeRequirementDto obeRequirementDto : obeRequirementDtoList) {
                List<ObeSubRequirementsDto> obeSubRequirementsDtoList = obeRequirementDto.getObeSubRequirements();
                for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                    if (obeSubRequirementsDtoMap.containsKey(obeSubRequirementsDto.getId())) {
                        Double degree = obeSubRequirementsDtoMap.get(obeSubRequirementsDto.getId()).stream().mapToDouble(x -> x.getMatrixDegree().doubleValue()).sum();
                        obeRequirementDto.setMatrixDegree(new BigDecimal(degree).setScale(2, BigDecimal.ROUND_HALF_UP));
                    }
                }
                Double degree = obeRequirementDtoList.stream().filter(x -> x.getMatrixDegree().doubleValue() > 0).mapToDouble(x -> x.getMatrixDegree().doubleValue()).min().orElse(0.0d);
                o.setMatrixDegree(new BigDecimal(degree).setScale(2, BigDecimal.ROUND_HALF_UP));
            }
        }
        return obeCourseRequirementDtoList;
    @ApiOperation(value = "毕业要求达成度学生报表-列表")
    @RequestMapping(value = "/requirements/student/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度学生报表", response = ScoreResult.class)})
    @Aac(auth = false)
    public Result requirementsStudentPage(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId,
                                          @ApiParam(value = "学号/姓名", required = true) @RequestParam String studentCodeOrName,
                                          @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                          @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(trExamStudentService.getRequirementsStudentPage(cultureProgramId, studentCodeOrName, pageNumber, pageSize));
    }

    @ApiOperation(value = "毕业要求达成度学生报表-查看详情")
    @RequestMapping(value = "/requirements/student/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度学生报表", response = ScoreResult.class)})
    @Aac(auth = false)
    public Result requirementsStudentReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId,
                                            @ApiParam(value = "学号", required = true) @RequestParam String studentCode,
                                            @ApiParam(value = "学期", required = true) @RequestParam Long semesterId) {
        return ResultUtil.ok(trExamStudentService.getRequirementsStudentReport(cultureProgramId, studentCode, semesterId));
    }
}
